/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.admin.ui.action;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.solutions.identity.IdentityProviderException;
import org.wso2.solutions.identity.admin.KeystoreUtilAdmin;
import org.wso2.solutions.identity.admin.RelyingPartyAdmin;
import org.wso2.solutions.identity.persistence.dataobject.RelyingPartyDO;

import java.io.File;
import java.io.FileInputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateParsingException;
import java.security.cert.X509Certificate;

public class UploadRelyingPartyCertificateAction extends ManagedAction {

    private static final long serialVersionUID = -2143108113462435214L;

    private static Log log = LogFactory
            .getLog(UploadRelyingPartyCertificateAction.class);

    private File certificateFile;

    private String certificateFileType;

    private String certificateFileName;

    public String execute() throws Exception {

        KeystoreUtilAdmin ksAdmin = new KeystoreUtilAdmin();
        String alias;

        if (this.certificateFile != null) {
            X509Certificate cert = null;
            try {
                CertificateFactory factory = CertificateFactory
                        .getInstance("X.509");
                cert = (X509Certificate) factory
                        .generateCertificate(new FileInputStream(
                                certificateFile));
            } catch (CertificateParsingException e) {
                this.addErrorMessage(getText("cert-invalid"));
                return ERROR;
            }
            
            if (ksAdmin.isCertExists(cert)) {
                this.addErrorMessage(getText("relying-party-exists"));
                return ERROR;
            }

            try {
                alias = ksAdmin.importCert(cert);
                RelyingPartyDO rp = new RelyingPartyDO();
                rp.setAlias(alias);
                rp.setHostName(alias);

                RelyingPartyAdmin admin = new RelyingPartyAdmin();
                admin.create(rp);

                return SUCCESS;
            } catch (IdentityProviderException e) {
                this.addErrorMessage(e.getMessage());
                log.error(e.getMessage(), e);
                return ERROR;
            }

        } else {
            this.addErrorMessage("no-cert-file");
            return ERROR;
        }

    }

    public File getCertificateFile() {
        return certificateFile;
    }

    public void setCertificateFile(File certificateFile) {
        this.certificateFile = certificateFile;
    }

    public String getCertificateFileContentType() {
        return certificateFileType;
    }

    public void setCertificateFileContentType(String certificateFileType) {
        this.certificateFileType = certificateFileType;
    }

    public String getCertificateFileFileName() {
        return certificateFileName;
    }

    public void setCertificateFileFileName(String certificateFileName) {
        this.certificateFileName = certificateFileName;
    }

}
