/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <httpext.h>
#include <httpfilt.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define POST_METHOD "post"

#define INTERNET_MAX_PATH_LENGTH        2048
#define INTERNET_MAX_SCHEME_LENGTH      32          // longest protocol name length
#define INTERNET_MAX_URL_LENGTH         (INTERNET_MAX_SCHEME_LENGTH \
                                        + sizeof("://") \
                                        + INTERNET_MAX_PATH_LENGTH)

#define EXTENSION_URL					"/test_folder/cspace_isapi.dll"
#define LOGON_URL						"/Logon"
#define LOGOFF_URL						"/Logoff"

#define FOLDER_NAME_HEADER				"CSPACE-FOLDER-NAME:"
#define ORIGINAL_URL					"CSPACE-ORI-URL:"
#define HTTP_FOLDER_NAME_HEADER			"HTTP_CSPACE_FOLDER_NAME"
#define HTTP_ORIGINAL_URL				"HTTP_CSPACE_ORI_URL"

#define HTML_ERROR_400          "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"  \
                                "<HTML><HEAD><TITLE>Bad request!</TITLE></HEAD>"                    \
                                "<BODY><H1>Bad request!</H1><DL><DD>\n"                             \
                                "Your browser (or proxy) sent a request that "                      \
                                "this server could not understand.</DL></DD></BODY></HTML>"

#define HTML_ERROR_404          "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"  \
                                "<HTML><HEAD><TITLE>Object not found!</TITLE></HEAD>"               \
                                "<BODY><H1>The requested URL was not found on this server"          \
                                "</H1><DL><DD>\nIf you entered the URL manually please check your"  \
                                "spelling and try again.</DL></DD></BODY></HTML>"

typedef struct config_s {
	char *name;
	char *value;
} config_t;

char *folders[] = {"/test/", "/test1/t/"};
const int no_folders = 2;

static int start_response(EXTENSION_CONTROL_BLOCK *pECB,
                                    int status,
                                    const char *reason,
                                    const char *const *header_names,
                                    const char *const *header_values,
                                    unsigned int num_of_headers);

static char *status_reason(int status);
int read_client (EXTENSION_CONTROL_BLOCK *pECB, char *buffer, size_t *size);
void WINAPI HandleExecUrlCompletion(EXTENSION_CONTROL_BLOCK *pECB, void * pContext, DWORD cbIO, DWORD dwError);

BOOL WINAPI DllMain(IN HINSTANCE hinstDll, IN DWORD dwReason, IN LPVOID lpvContext) 
{ 
	// Note that appropriate initialization and termination code 
	// would usually be written within the switch statement below. But because 
	// this example is very simple, no initialization and termination code is  needed. 
	switch(dwReason) { 
		case DLL_PROCESS_ATTACH : 
			break; 
		case DLL_PROCESS_DETACH : 
			break; 
	} 
	return TRUE; 
} 



BOOL WINAPI GetFilterVersion(PHTTP_FILTER_VERSION pVer) 
{
	DWORD rc = FALSE;
    ULONG http_filter_revision = HTTP_FILTER_REVISION;
    pVer->dwFilterVersion = pVer->dwServerFilterVersion;
    if (pVer->dwFilterVersion > http_filter_revision) {
        pVer->dwFilterVersion = http_filter_revision;
    }

	/*
	Receive notifictions when
		1. Server preprocessed the headers.
		2. Log 
		3. All the request coming in secure and none secure ports.
	*/
	pVer->dwFlags = (SF_NOTIFY_ORDER_LOW |
                    SF_NOTIFY_SECURE_PORT |
                    SF_NOTIFY_NONSECURE_PORT |
                    SF_NOTIFY_PREPROC_HEADERS |
                    SF_NOTIFY_LOG |
                    SF_NOTIFY_AUTH_COMPLETE
					);

	// Give a short discription about the module.
	strcpy(pVer->lpszFilterDesc, "axis2c filter");
	
	return TRUE;
}

DWORD WINAPI HttpFilterProc(
		PHTTP_FILTER_CONTEXT pfc,
		DWORD notificationType,
		LPVOID pvNotification)
{		

	int i = 0;
	size_t length = 0, j = 0;
	BOOL matched = TRUE;
	char org_url[INTERNET_MAX_URL_LENGTH] = {0};
	char redi_url[INTERNET_MAX_URL_LENGTH] = {0};
	int ret_val = INTERNET_MAX_URL_LENGTH;
	int temp = 0;
	switch (notificationType){
		case SF_NOTIFY_PREPROC_HEADERS:						
			if(temp = ((PHTTP_FILTER_PREPROC_HEADERS)pvNotification)->GetHeader(pfc, "url", org_url, &ret_val)){				
				for (i = 0; i < no_folders; i++) {
					length = strlen(folders[i]);
					matched = TRUE;
					for (j = 0; j < length; j++){
						if (folders[i][j] != org_url[j]) {
							matched = FALSE;
						}
					}
					if (matched) {
						strcpy(redi_url, EXTENSION_URL);
						//strcat(redi_url, (char *)(org_url + length));						
						if(!((PHTTP_FILTER_PREPROC_HEADERS)pvNotification)->SetHeader(pfc, "url", redi_url)){												
							return SF_STATUS_REQ_ERROR;
						}
						if(!((PHTTP_FILTER_PREPROC_HEADERS)pvNotification)->AddHeader(pfc, FOLDER_NAME_HEADER, folders[i])){
							return SF_STATUS_REQ_ERROR;
						}
						if(!((PHTTP_FILTER_PREPROC_HEADERS)pvNotification)->AddHeader(pfc, ORIGINAL_URL, org_url)){
							return SF_STATUS_REQ_ERROR;
						}
						return SF_STATUS_REQ_HANDLED_NOTIFICATION;
					}
				}
			}
			break;				
		default:
			break;
	}
	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}


BOOL WINAPI TerminateFilter(DWORD dwfalgs)
{
	return TRUE;	
}

BOOL init_module ()
{
	return TRUE;
}

BOOL WINAPI GetExtensionVersion(HSE_VERSION_INFO *pVer) 
{ 
	pVer->dwExtensionVersion = MAKELONG(HSE_VERSION_MINOR, HSE_VERSION_MAJOR); 

	strcpy(pVer->lpszExtensionDesc, "IIS for Testing"); 

	return TRUE; 
} 


DWORD WINAPI HttpExtensionProc(EXTENSION_CONTROL_BLOCK *pECB) 
{ 	
	size_t read_bytes = 0;
	char *buffer = NULL;
	char https[8];
	int buffer_size = 8;
	char *ori_url = NULL;
	int ret_val = 0;
	HSE_EXEC_URL_INFO *exec_url_info;

	if (_stricmp(pECB->lpszMethod, POST_METHOD)) {
		start_response(pECB, 403, NULL, NULL, NULL, 0);
		return HSE_STATUS_ERROR;
	}
	
	if (!pECB->GetServerVariable(pECB->ConnID, "HTTPS", https, &buffer_size)){
		return HSE_STATUS_ERROR;
	}

	if (_stricmp("ON", https)){
		start_response(pECB, 403, NULL, NULL, NULL, 0);
		return HSE_STATUS_ERROR;
	}

	buffer = malloc(sizeof(char) * (pECB->cbTotalBytes + 1));
	read_bytes = pECB->cbTotalBytes + 1;
	read_client(pECB, buffer, &read_bytes);

	if (strstr(buffer, "ABC")){
		start_response(pECB, 403, NULL, NULL, NULL, 0);
		return HSE_STATUS_ERROR;													
	} else {
		ori_url = malloc(INTERNET_MAX_URL_LENGTH * sizeof(char));
		ret_val = INTERNET_MAX_URL_LENGTH;
		if (!pECB->GetServerVariable(pECB->ConnID, HTTP_ORIGINAL_URL, ori_url, &ret_val)) {
			start_response(pECB, 403, NULL, NULL, NULL, 0);
			return HSE_STATUS_ERROR;													
		}
		exec_url_info = (HSE_EXEC_URL_INFO *) malloc(sizeof(HSE_EXEC_URL_INFO));
		exec_url_info->pszUrl = ori_url;				
		//We need to set the child headers here
		exec_url_info->pszChildHeaders = NULL;
		exec_url_info->dwExecUrlFlags = HSE_EXEC_URL_IGNORE_CURRENT_INTERCEPTOR;
		// Followng two are inherited from the paren request
		exec_url_info->pszMethod = NULL;
		exec_url_info->pEntity = NULL;
		exec_url_info->pUserInfo = NULL;

		if ( !pECB->ServerSupportFunction( pECB->ConnID,
                                      HSE_REQ_IO_COMPLETION,
                                      HandleExecUrlCompletion,
                                      NULL,
									  NULL )){
			start_response(pECB, 403, NULL, NULL, NULL, 0);
			return HSE_STATUS_ERROR;													
		}
		    
		/* Call ExecUrl */
		if (!pECB->ServerSupportFunction( pECB->ConnID,
										  HSE_REQ_EXEC_URL,
										  exec_url_info,
										  NULL,
										  NULL )){
	        start_response(pECB, 403, NULL, NULL, NULL, 0);
			return HSE_STATUS_ERROR;													
		}		
		/* Since EXEC_URL is asynchonous, we need to return STATUS_PENDING.*/
		return HSE_STATUS_PENDING;
	}
	return HSE_STATUS_SUCCESS; 
} 


void WINAPI HandleExecUrlCompletion(EXTENSION_CONTROL_BLOCK *pECB, void * pContext, DWORD cbIO, DWORD dwError)
{    
	/*Done*/
    pECB->ServerSupportFunction( pECB->ConnID, HSE_REQ_DONE_WITH_SESSION, NULL, NULL, NULL );
	return;
}

BOOL WINAPI TerminateExtension(DWORD dwFlags)
{
	return TRUE;
}

int read_client (EXTENSION_CONTROL_BLOCK *pECB, char *buffer, size_t *size)
{
	int read = 0;
	int need_to_read = 0;
	int temp = 0;
	if (pECB->cbAvailable == pECB->cbTotalBytes){
		if (*size >= pECB->cbAvailable){
			memcpy (buffer, pECB->lpbData, pECB->cbAvailable);						
		} else {
			memcpy (buffer, pECB->lpbData, pECB->cbAvailable);
		}
	} 
	else if (pECB->cbAvailable < pECB->cbTotalBytes){
		memcpy( buffer, pECB->lpbData, pECB->cbAvailable);
		need_to_read = pECB->cbTotalBytes - pECB->cbAvailable;
		while ( read != need_to_read){
			pECB->ReadClient(pECB->ConnID, buffer + read, &temp);
			read += temp;
		}
	}
	return 1;
}

int write_response(EXTENSION_CONTROL_BLOCK *pECB, const void *b, unsigned int l)
{    
    if (pECB && b) {
        if (l) {
            unsigned int written = 0;
            char *buf = (char *)b;
			// If couldn't write the data at onece try again until all the data is written.
            while (written < l) {
                DWORD try_to_write = l - written;
                if (!pECB->WriteClient(pECB->ConnID,
                                           buf + written, &try_to_write, 0)) {
                    return 0;
                }
                written += try_to_write;
            }
        }        
        return 1;
    }    
    return 0;
}


static char *status_reason(int status)
{
    static struct reasons {
        int status;
        char *reason;
    } *r, reasons[] = {
        { 100, "Continue" },
        { 101, "Switching Protocols" },
        { 200, "OK" },
        { 201, "Created" },
        { 202, "Accepted" },
        { 203, "Non-Authoritative Information" },
        { 204, "No Content" },
        { 205, "Reset Content" },
        { 206, "Partial Content" },
        { 300, "Multiple Choices" },
        { 301, "Moved Permanently" },
        { 302, "Moved Temporarily" },
        { 303, "See Other" },
        { 304, "Not Modified" },
        { 305, "Use Proxy" },
        { 400, "Bad Request" },
        { 401, "Unauthorized" },
        { 402, "Payment " },
        { 403, "Forbidden" },
        { 404, "Not Found" },
        { 405, "Method Not Allowed" },
        { 406, "Not Acceptable" },
        { 407, "Proxy Authentication Required" },
        { 408, "Request Timeout" },
        { 409, "Conflict" },
        { 410, "Gone" },
        { 411, "Length Required" },
        { 412, "Precondition Failed" },
        { 413, "Request Entity Too Large" },
        { 414, "Request-URI Too Long" },
        { 415, "Unsupported Media Type" },
        { 500, "Internal Server Error" },
        { 501, "Not Implemented" },
        { 502, "Bad Gateway" },
        { 503, "Service Unavailable" },
        { 504, "Gateway Timeout" },
        { 505, "HTTP Version Not Supported" },
        { 000, NULL}
    };

    r = reasons;
    while (r->status <= status)
        if (r->status == status)
            return r->reason;
        else
            r++;
    return "No Reason";
}

static int process_logon(EXTENSION_CONTROL_BLOCK *pECB)
{
	
	return FALSE;
}

static int process_logoff(EXTENSION_CONTROL_BLOCK *pECB)
{

	return FALSE;
}


static int start_response(EXTENSION_CONTROL_BLOCK *pECB,
                                    int status,
                                    const char *reason,
                                    const char *const *header_names,
                                    const char *const *header_values,
                                    unsigned int num_of_headers)
{
    static char crlf[3] = { (char)13, (char)10, '\0' };

    if (status < 100 || status > 1000) {        
        return FALSE;
    }

    if (pECB) {                
        size_t len_of_status;
        char *status_str;
        char *headers_str;

         /*
         * Create the status line
         */
        if (!reason) {
            reason = status_reason(status);
        }
        status_str = (char *)_alloca((6 + strlen(reason)) * sizeof(char));
        sprintf(status_str, "%d %s", status, reason);
        len_of_status = strlen(status_str);

		/*
		 * Create response headers string
		 */
		if (num_of_headers) {
			size_t i, len_of_headers;
			for (i = 0, len_of_headers = 0; i < num_of_headers; i++) {
				len_of_headers += strlen(header_names[i]);
				len_of_headers += strlen(header_values[i]);
				len_of_headers += 4;        /* extra for colon, space and crlf */
			}

			len_of_headers += 3;    /* crlf and terminating null char */
			headers_str = (char *)_alloca(len_of_headers * sizeof(char));
			headers_str[0] = '\0';

			for (i = 0; i < num_of_headers; i++) {
				strcat(headers_str, header_names[i]);
				strcat(headers_str, ": ");
				strcat(headers_str, header_values[i]);
				strcat(headers_str, crlf);
			}
			strcat(headers_str, crlf);
		}
		else {
			headers_str = crlf;
		}

        if (!pECB->ServerSupportFunction(pECB->ConnID,
                                             HSE_REQ_SEND_RESPONSE_HEADER,
                                             status_str,
                                             (LPDWORD) &len_of_status,
                                             (LPDWORD) headers_str)) {            
            return FALSE;
        }        
        return TRUE;

    }

    return FALSE;
}

BOOL create_cookie(char *buffer, int buf_size, char **names, 
				   char **values, int name_val, 
				   char *exp_date, char *domain, 
				   char *path, char *secure)
{
	int i = 0;
	int cur_buf_pos = 0;
	if (buffer && buf_size)
	{
		buffer[0] = '\0';
		if (name_val || exp_date || domain || path || secure){
			strcat(buffer, "Set-Cookie:");
			cur_buf_pos += (int)strlen("Set-Cookie:");
		} 
		else {
			return FALSE;
		}

		for(i = 0;i < name_val;i++){
			strcat(buffer + cur_buf_pos, names[i]);
			cur_buf_pos += (int)strlen(names[i] + 1);
			strcat(buffer + cur_buf_pos, ";");
			cur_buf_pos++;
		}	

		if (exp_date){
			sprintf(buffer + cur_buf_pos, "%s%s%s", "expires=", exp_date, ";");
			cur_buf_pos += (int)strlen("expires=") + (int)strlen(exp_date) + 1;
		}
		if (domain){
			sprintf(buffer + cur_buf_pos, "%s%s%s", "domain=", domain, ";");
			cur_buf_pos += (int)strlen("domain=") + (int)strlen(domain) + 1;
		}
		if (path){
			sprintf(buffer + cur_buf_pos, "%s%s%s", "path=", path, ";");
			cur_buf_pos += (int)strlen("path=") + (int)strlen(path) + 1;			
		}
		if (secure){
			strcat(buffer + cur_buf_pos, secure);
		}
	}
	return TRUE;
}