#ifndef _PROCESS_REQUEST_H_
#define _PROCESS_REQUEST_H_

#ifdef __cplusplus
extern "C" {
#endif

#ifndef FAIL
    #define FAIL 0
    #define SUCC !(FAIL)
#endif 

#define CARDSPACE_HEADER_STATE "auth_state"
#define CARDSPACE_STATE_SUCCESS "success"
#define CARDSPACE_STATE_FAIL "failure"

#define cspace_process_context_create \
        cspace_process_context_create_with_allocator 

typedef enum {
    PCLOG_EMERG = 0,
    PCLOG_ALERT,
    PCLOG_CRIT,
    PCLOG_ERR,
    PCLOG_WARNING,
    PCLOG_NOTICE,
    PCLOG_INFO,
    PCLOG_DEBUG,
} pc_log_level_t;

typedef struct allocator allocator_t;
typedef struct process_context process_context_t;

typedef void *(*malloc_cb_t)(size_t size, void *cb_ctx);
typedef void (*free_cb_t)(void *ptr, void *cb_ctx);

typedef void (*logger_cb_t)(const char *msg, pc_log_level_t level, void *cb_ctx);

typedef void (*set_header_cb_t)(char *ns_uri, char *key, char *val, 
                                void *container, void *cb_ctx);

int cspace_process_context_init(void);

process_context_t *cspace_process_context_create_default(void);
process_context_t *cspace_process_context_create_with_allocator(
                                        malloc_cb_t malloc_fn, void *mctx,
                                        free_cb_t free_fn, void *fctx);

void cspace_process_context_free(process_context_t *ctx);
void cspace_process_context_halt(void);

void cspace_process_context_set_logger(process_context_t *ctx,
                                       logger_cb_t logger, void *cb_ctx);

int cspace_process_context_set_key_file(process_context_t *ctx,
                                        const char* key_file);

int cspace_process_context_set_ca_file(process_context_t *ctx,
                                       const char* ca_file);

void cspace_process_context_set_header_callback(process_context_t *ctx,
                                                set_header_cb_t set_header_fn,
                                                void *container);

void cspace_process_context_set_header_callback_context(process_context_t *ctx,
                                                void *cb_ctx);

int cspace_process_request(process_context_t *ctx, const char *req);



#ifdef __cplusplus
}
#endif

#endif

