/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.relyingparty.servletfilter;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.net.URI;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.om.util.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.security.WSSConfig;
import org.apache.ws.security.components.crypto.X509NameTokenizer;
import org.apache.ws.security.util.DOM2Writer;
import org.w3c.dom.Element;
import org.wso2.solutions.identity.IdentityConstants;
import org.wso2.solutions.identity.relyingparty.RelyingPartyException;
import org.wso2.solutions.identity.relyingparty.TokenVerifier;
import org.wso2.solutions.identity.relyingparty.TokenVerifierConstants;

public class RelyingPartyFilter implements Filter {

    public static Log log = LogFactory.getLog(RelyingPartyFilter.class);

    private PrivateKey privateKey = null;

    private String IdPstoreFilePath = null;

    private String IdPStorePass = null;

    private String IdpStoreType = null;

    private String validatePolicy = null;
    
    private String issuerPolicy = null;

    /**
     * One array contains one DN name
     */
    private List[] blackList = null;

    /**
     * One array contains one DN name
     */
    private List[] whiteList = null;

    /**
     * jre/lib/security/cacert keystore's password
     */
    private String defaultStorePass = null;

    static {
        WSSConfig.getDefaultWSConfig();
    }

    FilterConfig filterConfig = null;

    public void doFilter(ServletRequest request, ServletResponse response,
            FilterChain chain) throws IOException, ServletException {
        try {

            String path = ((HttpServletRequest) request).getRequestURI();

            if (log.isDebugEnabled()) {
                log.debug("RequestURI : " + path);
            }

            String infoCardSignin = request.getParameter("InfoCardSignin");

            if (log.isDebugEnabled()) {
                log.debug("InfoCardSignin : " + infoCardSignin);
            }

            if (infoCardSignin != null && "Log in".equals(infoCardSignin)) {

                if (log.isDebugEnabled()) {
                    log.debug("InfoCardSignin=Log in");
                }

                String xmlToken = request.getParameter("xmlToken");
                if (xmlToken != null) {

                    TokenVerifier verifier = new TokenVerifier();
                    Element plainTokenElem = verifier.decryptToken(xmlToken,
                            privateKey);

                    boolean bSuccessInjected = false;
                    if (verifier.verifyDecryptedToken(plainTokenElem)) {
                        if (validateIssuerInfoPolicy(verifier)) {
                            bSuccessInjected = true;
                            injectDataToRequestOnSuccess(verifier, request);
                        }
                    }

                    if (bSuccessInjected == false) {
                        injectDataToRequestOnFailure(verifier, request);
                    }

                } else {
                    request.setAttribute(TokenVerifierConstants.ATTR_STATE,
                            TokenVerifierConstants.STATE_FAILURE);
                    request.setAttribute(TokenVerifierConstants.FAILURE_REASON,
                            TokenVerifierConstants.REASON_TOKEN_MISSING);
                }
            }
        } catch (Exception e) {
            log.error("Error in token verification", e);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            e.printStackTrace(new PrintWriter(baos));
            request.setAttribute(TokenVerifierConstants.ATTR_STATE,
                    TokenVerifierConstants.STATE_FAILURE);
            request.setAttribute(TokenVerifierConstants.FAILURE_REASON,
                    new String(baos.toByteArray()));
        }
        chain.doFilter(request, response);
    }

    /**
     * Validates issuer info
     * 
     * @param verifier
     * @return Whether issue validation successful or not.
     * @throws Exception
     */
    protected boolean validateIssuerInfoPolicy(TokenVerifier verifier)
            throws Exception {
        boolean validated = false;
        String issuerName = verifier.getIssuerName();

        if (IdentityConstants.SELF_ISSUED_ISSUER.equals(issuerName)) {
            
            if (issuerPolicy == null
                    || issuerPolicy.equals(TokenVerifierConstants.SELF_ONLY)
                    || issuerPolicy
                            .equals(TokenVerifierConstants.SELF_AND_MANGED)) {
                validated = true;
            }
        } else if(issuerPolicy.equals(TokenVerifierConstants.SELF_ONLY)){
            //not a self issued card when self only
            validated = false;
        } else if (validatePolicy.equals(TokenVerifierConstants.PROMISCUOUS)) {
            validated = true;
        } else if (validatePolicy.equals(TokenVerifierConstants.BLACK_LIST)) {
            doCertValidation(verifier);
            if (doBlackListCheck(verifier)) {
                validated = true;
            }
        } else if (validatePolicy.equals(TokenVerifierConstants.WHITE_LIST)) {
            doCertValidation(verifier);
            if (doWhiteListCheck(verifier)) {
                validated = true;
            }
        } else if (validatePolicy.equals(TokenVerifierConstants.CERT_VALIDATE)) {
            doCertValidation(verifier);
            validated = true;
        }

        return validated;
    }

    protected boolean validateMultipleClaimsPolicy(TokenVerifier verifier)
            throws RelyingPartyException {
        boolean validated = true;
        String multipleClaimsPolicy = filterConfig
                .getInitParameter(TokenVerifierConstants.MULTIVALUE_CLAIMS_POLICY);

        if (multipleClaimsPolicy == null
                || multipleClaimsPolicy
                        .equals(TokenVerifierConstants.MULTIVALUED_CLAIMS_NOT_ALLOWED)) {
            if (verifier.isMultipleValues()) {
                // Unexpected Multi-Valued Claim
                validated = false;
            }
        }
        return validated;
    }

    protected void injectDataToRequestOnFailure(TokenVerifier verifier,
            ServletRequest request) throws Exception {

        request.setAttribute(TokenVerifierConstants.SERVLET_ATTR_STATE,
                TokenVerifierConstants.STATE_FAILURE);
    }

    protected void injectDataToRequestOnSuccess(TokenVerifier verifier,
            ServletRequest request) throws Exception {

        request.setAttribute(TokenVerifierConstants.SERVLET_ATTR_STATE,
                TokenVerifierConstants.STATE_SUCCESS);

        String issuerInfo = getIssuerInfoString(verifier);
        if (issuerInfo != null) {
            request
                    .setAttribute(TokenVerifierConstants.ISSUER_INFO,
                            issuerInfo);
        }

        Iterator propertyEntry = verifier.getAttributeTable().entrySet()
                .iterator();
        while (propertyEntry.hasNext()) {
            Entry entry = (Entry) propertyEntry.next();
            String key = (String) entry.getKey();
            String value = (String) entry.getValue();
            request.setAttribute(key, value);
        }

    }

    protected String getIssuerInfoString(TokenVerifier verifier)
            throws Exception {
        String issuerInfo = null;
        OMFactory factory = OMAbstractFactory.getOMFactory();
        OMNamespace ns = factory.createOMNamespace(TokenVerifierConstants.NS,
                TokenVerifierConstants.PREFIX);

        List certficates = verifier.getCertificates();
        Element keyInfo = verifier.getKeyInfoElement();
        Iterator ite = certficates.iterator();
        boolean siginingSet = false;
        OMElement certificates = null;
        OMElement certElem = null;
        while (ite.hasNext()) {
            X509Certificate cert = (X509Certificate) ite.next();
            byte[] encodedCert = cert.getEncoded();
            String base64Encoded = Base64.encode(encodedCert);
            if (certificates == null) {
                certificates = factory.createOMElement(
                        TokenVerifierConstants.LN_CERTIFICATES, ns);
            }

            certElem = factory.createOMElement(
                    TokenVerifierConstants.LN_CERTIFICATE, ns);
            if (siginingSet == false) {
                certElem.addAttribute(TokenVerifierConstants.LN_SIGNING_CERT,
                        "true", null);
                siginingSet = true;
            }
            certElem.setText(base64Encoded);
            certificates.addChild(certElem);
        }

        OMElement omKeyInfo = null;
        if (keyInfo != null) {
            String value = DOM2Writer.nodeToString(keyInfo);
            XMLStreamReader parser = XMLInputFactory.newInstance()
                    .createXMLStreamReader(new StringReader(value));
            StAXOMBuilder builder = new StAXOMBuilder(parser);
            omKeyInfo = builder.getDocumentElement();

        }

        if (certificates != null) {
            issuerInfo = certificates.toString();
        }

        if (omKeyInfo != null) {
            issuerInfo = issuerInfo + omKeyInfo.toString();
        }

        return issuerInfo;
    }

    /**
     * This method read parameters from web.xml Step 1 : Read my private key
     * Step 2 : Read the token validation policy Step 3 : Read paramters that is
     * needed perform token validation
     * 
     * if(promiscuous) ->read nothing else if(blackList) ->read the blacklist
     * and keystore details else if (whiteList)->read the whitelist and keystor
     * details
     * 
     */
    public void init(FilterConfig config) throws ServletException {
        this.filterConfig = config;
        // Step 1: Reading my private key
        String keyStore = filterConfig
                .getInitParameter(TokenVerifierConstants.KEY_STORE);
        String storeType = filterConfig
                .getInitParameter(TokenVerifierConstants.STORE_TYPE);
        String storePass = filterConfig
                .getInitParameter(TokenVerifierConstants.STORE_PASS);
        String keyAlias = filterConfig
                .getInitParameter(TokenVerifierConstants.KEY_ALIAS);
        String keyPass = filterConfig
                .getInitParameter(TokenVerifierConstants.KEY_PASS);
        
        issuerPolicy = filterConfig
        .getInitParameter(TokenVerifierConstants.ISSUER_POLICY);

        try {
            KeyStore store = KeyStore.getInstance(storeType);
            String realPath = config.getServletContext().getRealPath(keyStore);
            store.load(new FileInputStream(realPath), storePass.toCharArray());
            privateKey = (PrivateKey) store.getKey(keyAlias, keyPass
                    .toCharArray());
        } catch (Exception e) {
            throw new ServletException("Cannot load the private key", e);
        }

        // Step 2: Reading Token validate policy -
        // prmiscuous/blacklist/whitelist/certvalidte
        validatePolicy = filterConfig
                .getInitParameter(TokenVerifierConstants.TOKEN_VALIDATE_POLICY);

        if (validatePolicy == null) {
            validatePolicy = TokenVerifierConstants.CERT_VALIDATE; //.PROMISCUOUS;
        }

        // Step 3: Reading paramters of each policy

        if (validatePolicy.equals(TokenVerifierConstants.BLACK_LIST)) {
            String value = filterConfig
                    .getInitParameter(TokenVerifierConstants.BLACK_LIST);
            if (value != null) {
                blackList = readBlackWhiteList(value);
            }
        }

        if (validatePolicy.equals(TokenVerifierConstants.WHITE_LIST)) {
            String value = filterConfig
                    .getInitParameter(TokenVerifierConstants.WHITE_LIST);
            if (value != null) {
                whiteList = readBlackWhiteList(value);
            }
        }

        if (validatePolicy.equals(TokenVerifierConstants.WHITE_LIST)
                || validatePolicy.equals(TokenVerifierConstants.BLACK_LIST)
                || validatePolicy.equals(TokenVerifierConstants.CERT_VALIDATE)) {
            IdPstoreFilePath = filterConfig
                    .getInitParameter(TokenVerifierConstants.TRUSTED_KEY_STORE);
            IdPStorePass = filterConfig
                    .getInitParameter(TokenVerifierConstants.TRUSTED_STORE_PASS);
            IdpStoreType = filterConfig
                    .getInitParameter(TokenVerifierConstants.TRUSTED_STORE_TYPE);

            defaultStorePass = filterConfig
                    .getInitParameter(TokenVerifierConstants.SYSTEM_KEY_STORE_PASS);

            if (defaultStorePass == null) {
                // assume that it hasn't been changed
                defaultStorePass = "changeit";
            }
        }
    }

    public void destroy() {
    }

    /**
     * This method is suppose to do the certificate validation. It should check
     * for four things
     * 
     * 1) Is today's day within the validity period?
     * 
     * 2) Is issuing CA trusted? - Check form system store - Our store
     * 
     * 3) Does issuers public key validate the signature? (Already done)
     * 
     * 4) Does the domain name specified in the server's DN match the server's
     * actual DN?
     */
    protected void doCertValidation(TokenVerifier tokenVerifer)
            throws Exception {

        boolean isCertValid = false;

        String tokenIssuerName = tokenVerifer.getIssuerName();
        URI uri = new URI(tokenIssuerName);
        String tokenIssuerHostName = uri.getHost();
        if (tokenIssuerHostName == null) {
            throw new RelyingPartyException("invalidIssuerName");
        }

        X509Certificate signedCert = tokenVerifer.getSigningCert();
        String certIssuerName = signedCert.getIssuerDN().getName();

        // validity period
        signedCert.checkValidity();

        // is Trusted? checking in System store.
        boolean isContained = false;
        try {
            String javaHome = System.getenv("JAVA_HOME");
            if (javaHome == null) {
                throw new Exception("Cannot find JAVA_HOME");
            }
            String relativePath = null;

            if (File.separator.equals("/")) {
                relativePath = TokenVerifierConstants.CACERTS_STORE_UNIX;
            } else {
                relativePath = TokenVerifierConstants.CACERTS_STORE_WIN;
            }
            String defaultKeyStore = javaHome + relativePath;

            FileInputStream is = new FileInputStream(defaultKeyStore);
            KeyStore sysKS = KeyStore.getInstance("JKS");
            sysKS.load(is, defaultStorePass.toCharArray());
            if (sysKS != null) {
                isContained = sysKS.containsAlias(certIssuerName);
            }
        } catch (Exception e) {
            throw new RelyingPartyException("errorLoadingTrustedKeystore", e);
        }

        // is Trusted? checking in our store.
        if (!isContained) {
            try {
                String ksPath = this.filterConfig.getServletContext()
                        .getRealPath(IdPstoreFilePath);
                FileInputStream is = new FileInputStream(ksPath);
                KeyStore trustedIdP = KeyStore.getInstance(IdpStoreType);
                trustedIdP.load(is, IdPStorePass.toCharArray());
                isContained = trustedIdP.containsAlias(tokenIssuerHostName);
            } catch (Exception e) {
                throw new RelyingPartyException(
                        "errorLoadingTrustedIdpKeystore", e);
                
            }
        }

        if (isContained == false) {
            throw new RelyingPartyException("certificateNotTrusted");

        }

        /*
         * if (isContained) { //is Subject dn match the issuer name extracted
         * from the token? String subjectCN = getCNOfSubject(signedCert);
         * if(subjectCN == null){ throw new
         * RelyingPartyException("nullSubjectCommonName"); }
         * if(subjectCN.equals(tokenIssuerHostName)== false){ throw new
         * RelyingPartyException("subjectDNMismatch"); } }
         */

    }

    protected boolean doBlackListCheck(TokenVerifier tokenVerifer)
            throws RelyingPartyException {
        boolean isGreenLight = true;

        X509Certificate cert = tokenVerifer.getSigningCert();
        if (cert == null) {
            throw new RelyingPartyException("noCertInToken");
        }

        if (blackList == null) {
            isGreenLight = true;
        } else {
            String value = cert.getIssuerDN().getName();
            List certDN = getDNOfIssuer(value);
            for (int i = 0; i < blackList.length; i++) {
                List issuerDN = blackList[i];
                if (certDN.equals(issuerDN)) {
                    isGreenLight = false;
                    break;
                }
            }
        }

        return isGreenLight;
    }

    protected boolean doWhiteListCheck(TokenVerifier tokenVerifer)
            throws RelyingPartyException {
        boolean isGreenLight = false;

        X509Certificate cert = tokenVerifer.getSigningCert();
        if (cert == null) {
            throw new RelyingPartyException("noCertInToken");
        }

        if (whiteList != null) {
            String inString = cert.getIssuerDN().getName();
            List certDN = getDNOfIssuer(inString);
            for (int i = 0; i < whiteList.length; i++) {
                List issuerDN = whiteList[i];
                if (certDN.equals(issuerDN)) {
                    isGreenLight = true;
                    break;
                }
            }
        }
        return isGreenLight;
    }

    protected List getDNOfIssuer(String inString) {

        X509NameTokenizer nmTokens = new X509NameTokenizer(inString);
        List lst = new ArrayList();

        while (nmTokens.hasMoreTokens()) {
            lst.add(nmTokens.nextToken());
        }
        Collections.sort(lst);
        return lst;
    }

    protected String getCNOfSubject(X509Certificate cert) {
        String dn = cert.getIssuerDN().getName();
        if (dn.contains("CN=")) {
            int beginIndex = dn.indexOf("CN=");
            int endIndex = dn.indexOf(",", beginIndex);
            String name = dn.substring(beginIndex + 3, endIndex).trim();
            return name;
        }
        return null;
    }
    
    private List[] readBlackWhiteList(String paramString) {
        List[] dnList = null;
        String[] array = paramString.split("\\},\\{");

        if (array != null) {
            if ((array.length > 1) && array[0].startsWith("{")) {
                StringBuffer buff = new StringBuffer(array[0]);
                buff.deleteCharAt(0);
                array[0] = buff.toString();

            }
            int lastIndex = array.length - 1;
            if ((array.length > 1) && array[lastIndex].endsWith("}")) {
                StringBuffer buff = new StringBuffer(array[lastIndex]);
                buff.deleteCharAt(buff.length() - 1);
                array[lastIndex] = buff.toString();
            }

            dnList = new List[array.length];
            for (int i = 0; i < array.length; i++) {
                List lst = getDNOfIssuer(array[i]);
                dnList[i] = lst;
            }

        }

        return dnList;
    }

}
