/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.action;

import java.io.OutputStream;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.struts2.StrutsStatics;
import org.w3c.dom.Element;
import org.wso2.solutions.identity.cards.CardIssuer;
import org.wso2.solutions.identity.user.ui.UIConstants;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;

public class DownloadInfoCardAction extends ActionSupport {

    private static final long serialVersionUID = -1545801495450399056L;

    private Element informationCard;

    private String selfIssuedPpid;

    public String execute() throws Exception {
        ActionContext context = ActionContext.getContext();
        Map session = context.getSession();
        String userName = (String) session.get(UIConstants.USER);
        String sessionPpid = (String) session.get(UIConstants.PPID);

        CardIssuer cardIssuer = new CardIssuer();
        HttpServletResponse response = (HttpServletResponse) context
                .get(StrutsStatics.HTTP_RESPONSE);
        response.setContentType("application/x-informationCard");
        response.setHeader("Content-Disposition",
                "attachment; filename=\"wso2infoCard.crd\"");
        response.setHeader("cache-control", "maxage=3");
        response.setHeader("Pragma", "public");
        Element card = null;
        
        String details = null;
        
        if (selfIssuedPpid != null) {
            card = cardIssuer.issueCardForSelfIssuedCard(selfIssuedPpid, true);
         } else if (sessionPpid != null) {
            card = cardIssuer.issueCardForSelfIssuedCard(sessionPpid, true);
        } else {
            card = cardIssuer.issueCardForUsername(userName, true);
        }
    
        OutputStream out = response.getOutputStream();
        TransformerFactory.newInstance().newTransformer().transform(
                new DOMSource(card), new StreamResult(out));
        out.flush();
        out.close();
        return NONE;
    }

    public Element getInformationCard() {
        return informationCard;
    }

    public void setSelfIssuedPpid(String selfIssuedPpids) {
        this.selfIssuedPpid = selfIssuedPpids;
    }

}
