/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.action;

import org.apache.axiom.om.util.UUIDGenerator;
import org.apache.struts2.StrutsStatics;
import org.wso2.solutions.identity.IdentityConstants;
import org.wso2.solutions.identity.UserStore;
import org.wso2.solutions.identity.admin.ClaimsAdmin;
import org.wso2.solutions.identity.admin.RegisteredInfoCardInfoAdmin;
import org.wso2.solutions.identity.persistence.dataobject.ClaimDO;
import org.wso2.solutions.identity.relyingparty.TokenVerifierConstants;
import org.wso2.solutions.identity.sts.IdentityProviderUtil;

import javax.servlet.http.HttpServletRequest;

import java.util.HashMap;

import com.opensymphony.xwork2.ActionContext;

public class InfoCardUserRegistrationSubmitAction extends ManagedAction {

    private static final long serialVersionUID = -8132077631173378913L;

    public String execute() throws Exception {
        ActionContext context = ActionContext.getContext();
        HttpServletRequest request = (HttpServletRequest) context
                .get(StrutsStatics.HTTP_REQUEST);
        String ppid = (String)request.getAttribute(IdentityConstants.CLAIM_PPID);
        if(ppid == null) {
            this.addErrorMessage(getText("no_selector"));
            return ERROR;
        }
        
        RegisteredInfoCardInfoAdmin admin = new RegisteredInfoCardInfoAdmin();
        if(admin.isRegistedInformationCard(ppid)) {
            this.addErrorMessage(getText("card_alredy_registered",
                    new String[] { IdentityProviderUtil
                            .getPPIDDisplayValue(ppid) }));
            return ERROR;
        }
        
        try {
            UserStore store = UserStore.getInstance();
            
            String userName = IdentityProviderUtil.getPPIDDisplayValue(ppid);
            store.getRealm().getUserStoreAdmin().addUser(userName, UUIDGenerator.getUUID());

            ClaimsAdmin ClaimsAdmin = new ClaimsAdmin();
            ClaimDO[] claimDOs = ClaimsAdmin.getAllMappedEnabledClaims();
            HashMap props = new HashMap();

            for (int i = 0; i < claimDOs.length; i++) {
                props.put(claimDOs[i].getUri(), (String) request.getAttribute(claimDOs[i].getUri()));
            }
            store.getRealm().getUserStoreAdmin().setUserProperties(userName, props);

            String issuerInfo = (String) request.getAttribute(TokenVerifierConstants.ISSUER_INFO);
            admin.registerNewInfoCardInformation(ppid, userName, issuerInfo);
            
            this.addInfoMessage(getText("user_successfully_added",
                    new String[] { userName }));
        } catch (RuntimeException e) {
            this.addActionError(e.getMessage());
            return ERROR;
        }
        
        return SUCCESS;
    }

}
