/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.action;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.solutions.identity.UserStore;
import org.wso2.solutions.identity.admin.ReportAdmin;
import org.wso2.solutions.identity.persistence.dataobject.ActionDO;
import org.wso2.solutions.identity.user.ui.UIConstants;

import java.util.Map;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.validator.annotations.RequiredStringValidator;
import com.opensymphony.xwork2.validator.annotations.Validation;

@Validation
public class LoginAction extends ManagedAction {

    private static final long serialVersionUID = 8009090071152629790L;

    private static Log log = LogFactory.getLog(LoginAction.class);

    private String password;

    private String username;

    public String execute() throws Exception {
        if (this.doLogin()) {
            Map session = ActionContext.getContext().getSession();
            session.put(UIConstants.USER, username);
            String msg = getText("successful_for", new String[] {username});
            ReportAdmin.record(username, ActionDO.ACTION_USER_LOG_IN_UT, msg);
            this.addInfoMessage(getText("login_successful", new String[] {username}));
            return SUCCESS;
        } else {
            String msg  = getText("invalid_user_password");
            addActionError(msg);
            ReportAdmin.record(username, ActionDO.ACTION_USER_FAILURE, msg);
            return ERROR;
        }
    }

    private boolean doLogin() {
        try {
            UserStore userStore = UserStore.getInstance();
            return userStore.authenticate(username, password);
        } catch (Exception e) {

            log.error("Exception in authenticating user : username = "
                    + username + ",password = ", e);
            return false;
        }

    }
    @RequiredStringValidator(message = "", key = "username")
    public String getUsername() {
        return username;
    }


    @RequiredStringValidator(message = "", key = "password")
    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public void setUsername(String username) {
        this.username = username;
    }

}
