/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package org.wso2.solutions.identity.user.ui.action;

import org.wso2.solutions.identity.user.ui.UIConstants;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;

public class ManagedAction extends ActionSupport {

    private static final long serialVersionUID = -912963022083874288L;
    
    private List errorMessages;
    
    private List infoMessages;
    
    protected void addErrorMessage(String msg) {
        ArrayList<Message> msgs = getMessageList();
        msgs.add(new Message(msg, Message.ERROR));
    }

    protected void addInfoMessage(String msg) {
        ArrayList<Message> msgs = getMessageList();
        msgs.add(new Message(msg, Message.INFO));
    }

    protected void loadMessages() {
        this.errorMessages = this.getMessages(Message.ERROR);
        this.infoMessages = this.getMessages(Message.INFO);
        this.resetMessages();        
    }
    
    protected void resetMessages() {
        ActionContext context = ActionContext.getContext();
        Map session = context.getSession();
        session.remove(UIConstants.MAIN_PAGE_MESSAGES);
    }

    public List<String> getMessages(int type) {
        ArrayList<Message> msgs = getMessageList();
        Iterator<Message> msgIt = msgs.iterator();
        ArrayList<String> retMsgs = new ArrayList<String>();
        while (msgIt.hasNext()) {
            Message msg = (Message) msgIt.next();
            if(msg.getType() == type) {
                retMsgs.add(msg.getMsg());
            }
        }
        return retMsgs;
    }
    
    
    private ArrayList<Message> getMessageList() {
        ActionContext context = ActionContext.getContext();
        Map session = context.getSession();
        ArrayList<Message> msgs;
        if(session.get(UIConstants.MAIN_PAGE_MESSAGES) != null) {
            msgs = (ArrayList<Message>)session.get(UIConstants.MAIN_PAGE_MESSAGES);
        } else {
            msgs = new ArrayList<Message>();
            session.put(UIConstants.MAIN_PAGE_MESSAGES, msgs);
        }
        return msgs;
    }

    class Message {
        public final static int ERROR = 1;
        public final static int INFO = 2;
        
        String msg;
        int type;
        
        private Message(String msg, int type) {
            this.msg = msg;
            this.type = type;
        }

        public String getMsg() {
            return msg;
        }

        public int getType() {
            return type;
        }
    }
    
    public List getErrorMessages() {
        return errorMessages;
    }

    public List getInfoMessages() {
        return infoMessages;
    }

}
