/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.action;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts2.StrutsStatics;
import org.wso2.solutions.identity.IdentityConstants;
import org.wso2.solutions.identity.admin.RegisteredInfoCardInfoAdmin;
import org.wso2.solutions.identity.admin.ReportAdmin;
import org.wso2.solutions.identity.persistence.dataobject.ActionDO;
import org.wso2.solutions.identity.relyingparty.TokenVerifierConstants;
import org.wso2.solutions.identity.sts.IdentityProviderUtil;
import org.wso2.solutions.identity.user.ui.UIConstants;

import com.opensymphony.xwork2.ActionContext;

public class RegisterInfoCardAction extends ManagedAction {

    private static final long serialVersionUID = 8882642949940574577L;

    public String execute() throws Exception {
        ActionContext context = ActionContext.getContext();
        HttpServletRequest request = (HttpServletRequest) context
                .get(StrutsStatics.HTTP_REQUEST);

        Map session = context.getSession();
        String user = (String) session.get(UIConstants.USER);
        String ppid = (String) request
                .getAttribute(IdentityConstants.CLAIM_PPID);
      if(ppid == null) {
            addErrorMessage("Failure: submitting an information card");
            return ERROR;
        }
        String issuerInfo = (String) request
                .getAttribute(TokenVerifierConstants.ISSUER_INFO);

        RegisteredInfoCardInfoAdmin admin = new RegisteredInfoCardInfoAdmin();
        
        if(!admin.isRegistedInformationCard(ppid)) {
            //TODO: what are the chances of user picking a ppid as a username ????
            String primaryUserName = admin.extractPrimaryUserName(user);
            if(primaryUserName == null){
                admin.registerNewInfoCardInformation(ppid, user, issuerInfo);
                ReportAdmin.record(user, ActionDO.ACTION_USER_REGISTER, "PPID="+ppid);
            }else{
                admin.registerNewInfoCardInformation(ppid, primaryUserName, issuerInfo);
                ReportAdmin.record(primaryUserName, ActionDO.ACTION_USER_REGISTER, "PPID="+ppid);
            }
        } else {
            /*
             * This ERROR state will take the user back to the main page.
             * If someone tries to register the same card again
             * it is already available in list of registered cards.
             * 
             * If someone tried to register a card with a ppid of another user,
             * he/she will not see anything and it is good to leave that user 
             * confused :P :-)
             */
            String msg = getText("register-card-already-exists",
                    new String[] { IdentityProviderUtil
                    .getPPIDDisplayValue(ppid) });
            addErrorMessage(msg);
            ReportAdmin.record(user, ActionDO.ACTION_USER_FAILURE, msg);
            return ERROR;
        }
        return SUCCESS;
    }

}
